@@ -1,6 +1,6 @@
 ;;; prv-emacs.el --- GNU Emacs specific code for preview.el
 
-;; Copyright (C) 2001  Free Software Foundation, Inc.
+;; Copyright (C) 2001, 02, 03, 04, 05  Free Software Foundation, Inc.
 
 ;; Author: David Kastrup
 ;; Keywords: convenience, tex, wp
@@ -26,23 +26,13 @@
 
 ;;; Code:
 
+(require 'tex-site)
+(require 'tex)
+(require 'latex)
+
 (defvar preview-compatibility-macros nil
   "List of macros only present when compiling/loading.")
 
-(defimage preview-nonready-icon ((:type xpm :file "prevwork.xpm" :ascent 90)
-				 (:type xbm :file "prevwork.xbm" :ascent
-					90))
-  "The symbol used for previews to be generated.")
-
-(defimage preview-error-icon ((:type xpm :file "preverr.xpm" :ascent 90)
-			      (:type xbm :file "preverr.xbm" :ascent
-				     90))
-  "The symbol used for PostScript errors.")
-
-(defimage preview-icon ((:type xpm :file "preview.xpm" :ascent 75)
-			(:type xbm :file "preview.xbm" :ascent 75))
-  "The symbol used for an open preview.")
-
 (defcustom preview-transparent-color '(highlight :background)
   "Color to appear transparent in previews.
 Set this to something unusual when using `preview-transparent-border',
@@ -58,7 +48,11 @@ to the default background in most other cases."
 			(const :tag "Foreground" :value :foreground))))
   :group 'preview-appearance)
 
-(defcustom preview-transparent-border 1.5
+;;; Note that the following default introduces a border only when
+;;; Emacs blinks politely when point is on an image (the tested
+;;; unrelated function was introduced at about the time image blinking
+;;; became tolerable).
+(defcustom preview-transparent-border (unless (fboundp 'posn-object-x-y) 1.5)
   "Width of transparent border for previews in pt.
 Setting this to a numeric value will add a border of
 `preview-transparent-color' around images, and will turn
@@ -86,13 +80,34 @@ Consults `preview-transparent-color'."
 			  (nth 1 preview-transparent-color)
 			  'default)))))
 
-(defmacro preview-create-icon (file type ascent)
-  "Create an icon from FILE, image TYPE and ASCENT."
-  `(list 'image
-	 :file ,file
-	 :type ,type
-	 :ascent ,ascent
-	 :heuristic-mask (if preview-transparent-border t (preview-get-heuristic-mask))))
+(defsubst preview-create-icon-1 (file type ascent border)
+  `(image
+    :file ,file
+    :type ,type
+    :ascent ,ascent
+    ,@(and border
+	   '(:mask (heuristic t)))))
+
+(defun preview-create-icon (file type ascent border)
+  "Create an icon from FILE, image TYPE, ASCENT and BORDER."
+  (list
+   (preview-create-icon-1 file type ascent border)
+   file type ascent border))
+
+(put 'preview-filter-specs :type
+     #'(lambda (keyword value &rest args)
+	 (if (image-type-available-p value)
+	     `(image :type ,value
+		     ,@(preview-filter-specs-1 args))
+	   (throw 'preview-filter-specs nil))))
+
+;; No defcustom here: does not seem to make sense.
+
+(defvar preview-tb-icon-specs
+  '((:type xpm :file "prvtex24.xpm")
+    (:type xbm :file "prvtex24.xbm")))
+
+(defvar preview-tb-icon nil)
 
 (defun preview-add-urgentization (fun ov &rest rest)
   "Cause FUN (function call form) to be called when redisplayed.
@@ -110,9 +125,9 @@ if there was any urgentization."
     (when (eq (car dispro) 'when)
       (prog1
 	  (car (cdr dispro))
-	  (overlay-put ov 'display (cdr (cdr dispro)))))))
+	(overlay-put ov 'display (cdr (cdr dispro)))))))
 
-(defmacro preview-nonready-copy ()
+(defsubst preview-icon-copy (icon)
   "Prepare a later call of `preview-replace-active-icon'."
 
   ;; This is just a GNU Emacs specific efficiency hack because it
@@ -125,12 +140,13 @@ if there was any urgentization."
   ;; modifying the string in the strings property would change that
   ;; glyph automatically.
 
-  '(cons 'image (cdr preview-nonready-icon)))
+  (cons 'image (cdr icon)))
 
-(defmacro preview-replace-active-icon (ov replacement)
+(defsubst preview-replace-active-icon (ov replacement)
   "Replace the active Icon in OV by REPLACEMENT, another icon."
-  `(setcdr (overlay-get ,ov 'preview-image)
-	   (cdr ,replacement)))
+  (let ((img (overlay-get ov 'preview-image)))
+    (setcdr (car img) (cdar replacement))
+    (setcdr img (cdr replacement))))
 
 (defvar preview-button-1 [mouse-2])
 (defvar preview-button-2 [mouse-3])
@@ -160,36 +176,6 @@ are functions to call on preview's clicks."
 	    'keymap resmap)
 	'resmap)))
 
-(defun preview-int-bb (bb)
-  "Make integer bounding box from possibly float BB."
-  ;; Due to a bug in earlier Emacs versions, we make this a list instead
-  ;; of a vector
-  (when bb
-    (list
-     (floor (aref bb 0))
-     (floor (aref bb 1))
-     (ceiling (aref bb 2))
-     (ceiling (aref bb 3)))))
-
-(defun preview-ps-image (filename scale &optional box)
-  "Place a PostScript image directly by Emacs.
-This uses Emacs built-in PostScript image support for
-rendering the preview image in EPS file FILENAME, with
-a scale factor of SCALE indicating the relation of desired
-image size on-screen to the size the PostScript code
-specifies.  If BOX is present, it is the bounding box info."
-  (unless box
-    (setq box (preview-extract-bb filename)))
-;; should the following 2 be rather intbb?
-  (create-image filename 'postscript nil
-		:pt-width (round
-			   (* scale (- (aref box 2) (aref box 0))))
-		:pt-height (round
-			    (* scale (- (aref box 3) (aref box 1))))
-		:bounding-box (preview-int-bb box)
-		:ascent (preview-ascent-from-bb box)
-		:heuristic-mask (if preview-transparent-border t (preview-get-heuristic-mask))))
-
 (defvar preview-overlay nil)
 
 (put 'preview-overlay
@@ -381,21 +367,26 @@ purposes."
 	   event
 	 (posn-window (event-start event))))))
 
+(defsubst preview-buffer-recode-system (base)
+  "This is supposed to translate unrepresentable base encodings
+ into something that can be used safely for byte streams in the
+ run buffer.  A noop for Emacs."
+  base)
+
 (defun preview-mode-setup ()
   "Setup proper buffer hooks and behavior for previews."
+  (set (make-local-variable 'desktop-save-buffer)
+       #'desktop-buffer-preview-misc-data)
   (add-hook 'pre-command-hook #'preview-mark-point nil t)
   (add-hook 'post-command-hook #'preview-move-point nil t)
   (easy-menu-add preview-menu LaTeX-mode-map)
-  ;;The following is a crock, but it does not load tool-bar-mode in case
-  ;;nobody else does, and it should work with any Emacs-21.  Fixing
-  ;;up the ascent of preview-icon is butt-ugly, but faster than using
-  ;;another defimage.
-  (define-key LaTeX-mode-map [tool-bar preview]
-    `(menu-item "Preview at point" preview-at-point
-		:image ,(let ((image (copy-sequence preview-icon)))
-			  (plist-put (cdr image) :ascent 50)
-			  image)
-		:help "Preview on/off at point"))
+  (unless preview-tb-icon
+    (setq preview-tb-icon (preview-filter-specs preview-tb-icon-specs)))
+  (when preview-tb-icon
+    (define-key LaTeX-mode-map [tool-bar preview]
+      `(menu-item "Preview at point" preview-at-point
+		  :image ,preview-tb-icon
+		  :help "Preview on/off at point")))
   (when buffer-file-name
     (let* ((filename (expand-file-name buffer-file-name))
 	   format-cons)
@@ -450,34 +441,30 @@ overlays not in the active window."
   (let (newlist (pt (point)))
     (setq preview-temporary-opened
 	  (dolist (ov preview-temporary-opened newlist)
-	    (if (catch 'keep
-		  (unless (and (overlay-buffer ov)
-			       (eq (overlay-get ov 'preview-state) 'inactive))
-		    (throw 'keep nil))
-		  (unless (eq (overlay-buffer ov) (current-buffer))
-		    (throw 'keep t))
-		  (when (and (> pt (overlay-start ov))
-			     (< pt (overlay-end ov)))
-		    (throw 'keep t))
-		  (preview-toggle ov t)
-		  nil)
-		(push ov newlist))))
+	    (and (overlay-buffer ov)
+		 (eq (overlay-get ov 'preview-state) 'inactive)
+		 (if (and (eq (overlay-buffer ov) (current-buffer))
+			  (or (<= pt (overlay-start ov))
+			      (>= pt (overlay-end ov))))
+		     (preview-toggle ov t)
+		   (push ov newlist)))))
     (if (or disable-point-adjustment
 	    global-disable-point-adjustment
 	    (preview-auto-reveal-p preview-auto-reveal))
 	(preview-open-overlays (overlays-at pt))
       (let ((backward (and (eq (marker-buffer preview-marker) (current-buffer))
-			   (< pt (marker-position preview-marker)))))
-	(while (catch 'loop
-		 (dolist (ovr (overlays-at pt))
-		   (when (and
-			  (eq (overlay-get ovr 'preview-state) 'active)
-			  (> pt (overlay-start ovr)))
+			   (< pt (marker-position preview-marker))))
+	    (lst (overlays-at pt)))
+	(while lst
+	  (setq lst
+		(if (and
+		     (eq (overlay-get (car lst) 'preview-state) 'active)
+		     (> pt (overlay-start (car lst))))
+		    (overlays-at
 		     (setq pt (if backward
-				  (overlay-start ovr)
-				(overlay-end ovr)))
-		     (throw 'loop t))))
-	  nil)
+				  (overlay-start (car lst))
+				(overlay-end (car lst)))))
+		  (cdr lst))))
 	(goto-char pt)))))
 
 (defun preview-open-overlays (list &optional pos)
@@ -509,37 +496,54 @@ overlays not in the active window."
 	 (ad-activate 'replace-highlight))
   :initialize #'custom-initialize-reset)
 
-(defun preview-inherited-face-attribute (face attribute &optional
-					      fallbacks)
-  "Fetch face attribute while adhering to inheritance.
+;; Check whether the four-argument form of `face-attribute' exists.
+;; If not, we will get a `wrong-number-of-arguments' error thrown.
+;; Use `defun' instead of `defsubst' here so that the decision may be
+;; reverted at load time if you are compiling with one Emacs and using
+;; another.
+(if (condition-case nil
+	(progn
+	  (face-attribute 'default :height nil nil)
+	  t)
+      (wrong-number-of-arguments nil))
+
+    (defun preview-inherited-face-attribute (face attribute &optional inherit)
+      "Fetch face attribute while adhering to inheritance.
+This searches FACE for an ATTRIBUTE, using INHERIT
+for resolving unspecified or relative specs.  See the fourth
+argument of function `face-attribute' for details."
+      (face-attribute face attribute nil inherit))
+
+  (defun preview-inherited-face-attribute (face attribute &optional inherit)
+    "Fetch face attribute while adhering to inheritance.
 This searches FACE for an ATTRIBUTE.  If it is 'unspecified,
-first inheritance is consulted (if FALLBACKS is non-NIL), then
-FALLBACKS is searched if it is a face or a list of faces.
+first inheritance is consulted (if INHERIT is non-NIL), then
+INHERIT is searched if it is a face or a list of faces.
 Relative specs are evaluated recursively until they get absolute or
 are not resolvable.  Relative specs are float values."
-  (let ((value (face-attribute face attribute)))
-    (when fallbacks
-      (setq fallbacks
-	    (append
-	     (let ((ancestors (face-attribute face :inherit)))
-	       (cond ((facep ancestors) (list ancestors))
-		     ((consp ancestors) ancestors)))
-	     (cond ((facep fallbacks) (list fallbacks))
-		   ((consp fallbacks) fallbacks)))))
-    (cond ((null fallbacks) value)
-	  ((floatp value)
-	   (let ((avalue
-		  (preview-inherited-face-attribute
-		   (car fallbacks) attribute (or (cdr fallbacks) t))))
-	     (cond ((integerp avalue)
-		    (round (* avalue value)))
-		   ((floatp avalue)
-		    (* value avalue))
-		   (t value))))
-	  ((eq value 'unspecified)
-	   (preview-inherited-face-attribute
-	    (car fallbacks) attribute (or (cdr fallbacks) t)))
-	  (t value))))
+    (let ((value (face-attribute face attribute)))
+      (when inherit
+	(setq inherit
+	      (append
+	       (let ((ancestors (face-attribute face :inherit)))
+		 (cond ((facep ancestors) (list ancestors))
+		       ((consp ancestors) ancestors)))
+	       (cond ((facep inherit) (list inherit))
+		     ((consp inherit) inherit)))))
+      (cond ((null inherit) value)
+	    ((floatp value)
+	     (let ((avalue
+		    (preview-inherited-face-attribute
+		     (car inherit) attribute (or (cdr inherit) t))))
+	       (cond ((integerp avalue)
+		      (round (* avalue value)))
+		     ((floatp avalue)
+		      (* value avalue))
+		     (t value))))
+	    ((eq value 'unspecified)
+	     (preview-inherited-face-attribute
+	      (car inherit) attribute (or (cdr inherit) t)))
+	    (t value)))))
 
 (defun preview-get-colors ()
   "Return colors from the current display.
@@ -560,26 +564,22 @@ The fourth value is the transparent border thickness."
     (unless (and (numberp preview-transparent-border)
 		 (consp mask) (integerp (car mask)))
       (setq mask nil))
-    (vector bg fg mask (and mask preview-transparent-border))))
+    (vector bg fg mask preview-transparent-border)))
 
 (defmacro preview-mark-active ()
   "Return t if the mark is active."
   'mark-active)
 
-(defun preview-export-image (image)
-  "Format an IMAGE into something printable."
-  (let ((plist (cdr image)))
-    (list (plist-get plist :file)
-	  (plist-get plist :type)
-	  (plist-get plist :ascent))))
-
 (defun preview-import-image (image)
   "Convert the printable IMAGE rendition back to an image."
   (if (eq (car image) 'image)
       image
-    (preview-create-icon (nth 0 image)
-			 (nth 1 image)
-			 (nth 2 image))))
+    (preview-create-icon-1 (nth 0 image)
+			   (nth 1 image)
+			   (nth 2 image)
+			   (if (< (length image) 4)
+			       (preview-get-heuristic-mask)
+			     (nth 3 image)))))
 
 (defsubst preview-supports-image-type (imagetype)
   "Check if IMAGETYPE is supported."
