@@ -1,6 +1,7 @@
 ;;; prv-xemacs.el --- XEmacs support for preview-latex
 
-;; Copyright (C) 2001  Free Software Foundation, Inc.
+;; Copyright (C) 2001, 02, 03, 04, 05 Free Software Foundation, Inc.
+;; Parts (C)  2002 Nick Alcock.
 
 ;; Author: David Kastrup
 ;; Keywords: convenience, tex, wp
@@ -27,6 +28,9 @@
 ;;; Code:
 
 (require 'overlay)
+(require 'tex-site)
+(require 'tex)
+(require 'latex)
 
 ;; Compatibility macros and functions.
 
@@ -146,8 +150,12 @@ The message is displayed with label `progress'; see `display-message'."
 ;; In time, this will hopefully all migrate into XEmacs.
 
 ; XEmacs's `add-to-list' takes only two arguments.
-(defun add-to-list (list-var element &optional append)
-  "Add to the value of LIST-VAR the element ELEMENT if it isn't there yet.
+(condition-case nil
+    (let (check)
+      (add-to-list 'check t t))
+  (error
+   (defun add-to-list (list-var element &optional append)
+     "Add to the value of LIST-VAR the element ELEMENT if it isn't there yet.
 The test for presence of ELEMENT is done with `equal'.
 If ELEMENT is added, it is added at the beginning of the list,
 unless the optional argument APPEND is non-nil, in which case
@@ -158,32 +166,14 @@ until a certain package is loaded, you should put the call to `add-to-list'
 into a hook function that will be run only after loading the package.
 `eval-after-load' provides one way to do this.  In some cases
 other hooks, such as major mode hooks, can do the job."
-  (if (member element (symbol-value list-var))
-      (symbol-value list-var)
-    (set list-var
-         (if append
-             (append (symbol-value list-var) (list element))
-           (cons element (symbol-value list-var))))))
-
-(defvar null-device (or (and (boundp 'grep-null-device)
-			     grep-null-device)
-			(if (memq system-type '(ms-dos windows-nt))
-			    "NUL"
-			  "/dev/null"))
-  "The system null device.")
-
-;; We have to pick the image-specifier's inst-list apart by hand.  This code
-;; won't work if there are multiple valid image types in a single glyph in a
-;; single (locale * domain): but we don't care, because only the first of those
-;; will ever be rendered (and make-glyph probably can't create such images
-;; anyway).
-;; How horrible.
-
-(defmacro glyph-image-type (glyph)
-  "Return GLYPH's image type."
-  `(map-specifier (glyph-image ,glyph)
-                  #'(lambda (spec locale inst-list null)
-                      (elt (cdr (assoc (list (device-type)) inst-list)) 0))))
+     (if (member element (symbol-value list-var))
+	 (symbol-value list-var)
+       (set list-var
+	    (if append
+		(append (symbol-value list-var) (list element))
+	      (cons element (symbol-value list-var))))))))
+     
+(defvar preview-transparent-border)
 
 ;; Images.
 
@@ -192,7 +182,12 @@ other hooks, such as major mode hooks, can do the job."
 ;; Argh, dired breaks :file :(
 ;; This is a temporary kludge to get around that until a fixed dired
 ;; or a fixed XEmacs is released.
-(defsubst preview-create-icon (file type ascent)
+
+(defsubst preview-supports-image-type (imagetype)
+  "Return whether IMAGETYPE is supported by XEmacs."
+  (memq imagetype (image-instantiator-format-list)))
+
+(defun preview-create-icon-1 (file type ascent)
   "Create an icon from FILE, image TYPE and ASCENT."
   (let ((glyph
 	 (make-glyph
@@ -204,69 +199,37 @@ other hooks, such as major mode hooks, can do the job."
     (set-glyph-baseline glyph ascent)
     glyph))
 
-(defvar preview-nonready-icon
-  (let ((glyph-xpm-filename (locate-data-file "prevwork.xpm"))
-        (glyph-xbm-filename (locate-data-file "prevwork.xbm")))
-    (if (not glyph-xpm-filename)
-        (cerror 'file-error "Image not installed" "prevwork.xpm"))
-    (if (not glyph-xbm-filename)
-        (cerror 'file-error "Image not installed" "prevwork.xbm"))
-    (let ((glyph
-           (make-glyph
-            (list
-             `[xpm :file ,glyph-xpm-filename]
-             `[xbm :file ,glyph-xbm-filename]))))
-      (set-glyph-baseline glyph 90)
-      glyph))
-  "The symbol used for previews to be generated.")
+(defun preview-create-icon (file type ascent border)
+  "Create an icon from FILE, image TYPE, ASCENT and BORDER."
+  (list
+   (preview-create-icon-1 file type ascent)
+   file type ascent border))
 
-(defvar preview-error-icon
-  (let ((glyph-xpm-filename (locate-data-file "preverr.xpm"))
-        (glyph-xbm-filename (locate-data-file "preverr.xbm")))
-    (if (not glyph-xpm-filename)
-        (cerror 'file-error "Image not installed" "preverr.xpm"))
-    (if (not glyph-xbm-filename)
-        (cerror 'file-error "Image not installed" "preverr.xbm"))
-    (let ((glyph
-           (make-glyph
-            (list
-             `[xpm :file ,glyph-xpm-filename]
-           `[xbm :file ,glyph-xbm-filename]))))
-      (set-glyph-baseline glyph 90)
-      glyph))
-  "The symbol used for PostScript errors.")
+(defvar preview-ascent-spec)
 
-(defvar preview-icon
-  (let ((glyph-xpm-filename (locate-data-file "preview.xpm"))
-        (glyph-xbm-filename (locate-data-file "preview.xbm")))
-    (if (not glyph-xpm-filename)
-        (cerror 'file-error "Image not installed" "preview.xpm"))
-    (if (not glyph-xbm-filename)
-        (cerror 'file-error "Image not installed" "preview.xbm"))
-    (let ((glyph
-           (make-glyph
-            (list
-             `[xpm :file ,glyph-xpm-filename]
-             `[xbm :file ,glyph-xbm-filename]))))
-      (set-glyph-baseline glyph 75)
-      glyph))
-  "The symbol used for an open preview.")
+(put 'preview-filter-specs :type
+     #'(lambda (keyword value &rest args)
+	 (if (preview-supports-image-type value)
+	     (let* (preview-ascent-spec
+		    (glyph (make-glyph `[,value
+					 ,@(preview-filter-specs-1 args)])))
+	       (when preview-ascent-spec
+		 (set-glyph-baseline glyph preview-ascent-spec))
+	       glyph)
+	   (throw 'preview-filter-specs nil))))
 
-(defvar preview-tb-icon
-  (let ((glyph-xpm-filename (locate-data-file "preview-cap-up.xpm"))
-        (glyph-xbm-filename (locate-data-file "preview.xbm")))
-    (if (not glyph-xpm-filename)
-        (cerror 'file-error "Image not installed" "preview-cap-up.xpm"))
-    (if (not glyph-xbm-filename)
-        (cerror 'file-error "Image not installed" "preview.xbm"))
-    (let ((glyph
-           (make-glyph
-            (list
-             `[xpm :file ,glyph-xpm-filename]
-             `[xbm :file ,glyph-xbm-filename]))))
-      (set-glyph-baseline glyph 75)
-      glyph))
-  "The symbol used for the toolbar button.")
+(put 'preview-filter-specs :ascent
+     #'(lambda (keyword value &rest args)
+	 (setq preview-ascent-spec value)
+	 (preview-filter-specs-1 args)))
+
+;; No defcustom here: does not seem to make sense.
+
+(defvar preview-tb-icon-specs
+  '((:type xpm :file "prvtex-cap-up.xpm" :ascent 75)
+    (:type xbm :file "prvtex24.xbm" :ascent 75)))
+
+(defvar preview-tb-icon nil)
 
 ;; Image frobbing.
 
@@ -289,18 +252,17 @@ if there was any urgentization."
   (prog1 (list (extent-property ov 'initial-redisplay-function) ov)
     (set-extent-initial-redisplay-function ov nil)))
 
-(defmacro preview-nonready-copy ()
+(defsubst preview-icon-copy (icon)
   "Prepare for a later call of `preview-replace-active-icon'."
-  'preview-nonready-icon)
+  icon)
 
-(defmacro preview-replace-active-icon (ov replacement)
+(defsubst preview-replace-active-icon (ov replacement)
   "Replace the active Icon in OV by REPLACEMENT, another icon."
-  `(let ((replacement ,replacement))
-     (set-extent-property ,ov 'preview-image replacement)
-     (add-text-properties 0 1 (list 'end-glyph replacement)
-			  (car (extent-property ,ov 'strings)))
-     (if (eq (extent-property ,ov 'preview-state) 'active)
-	 (set-extent-end-glyph ,ov replacement))))
+  (set-extent-property ov 'preview-image replacement)
+  (add-text-properties 0 1 (list 'end-glyph (car replacement))
+		       (car (extent-property ov 'strings)))
+  (if (eq (extent-property ov 'preview-state) 'active)
+      (set-extent-end-glyph ov (car replacement))))
 
 (defvar preview-button-1 'button2)
 (defvar preview-button-2 'button3)
@@ -363,18 +325,6 @@ are functions to call on preview's clicks."
   (and (eq ov (event-glyph-extent (mouse-position-as-motion-event)))
        (extent-property ov 'preview-balloon-help)))
 
-(defun preview-ps-image (filename scale &optional box)
-  "Place a PostScript image directly by Emacs.
-This uses XEmacs built-in PostScript image support for
-rendering the preview image in EPS file FILENAME, with
-a scale factor of SCALE indicating the relation of desired
-image size on-screen to the size the PostScript code
-specifies.  If BOX is present, it is the bounding box info.
-
-Since there is, as yet, no such support, this is stubbed out.
-This will not be so forever."
-  (error 'image-conversion-error "PostScript images are not supported."))
-
 ;; Most of the changes to this are junking the use of overlays;
 ;; a bit of it is different, and there's a little extra paranoia.
 
@@ -467,19 +417,48 @@ Pure borderless black-on-white will return quadruple NIL."
         (setq fg nil))
     (vector bg fg nil nil)))
 
-(defcustom preview-use-balloon-help t
+(defcustom preview-use-balloon-help nil
   "*Is balloon help enabled in preview-latex?"
   :group 'preview-appearance
   :type 'boolean)
 
-(defvar preview-icon-toolbar-button
-  (vector (list preview-tb-icon)
-	  #'preview-at-point
-	  t
-	  "Preview on/off at point"))
+(defcustom preview-buffer-recoding-alist
+  (if (and (= emacs-major-version 21)
+	   (< emacs-minor-version 5))
+      '((utf-8-unix . raw-text-unix)
+	(utf-8-dos . raw-text-dos)
+	(utf-8-mac . raw-text-mac)
+	(utf-8 . raw-text)))
+  "Translate buffer encodings into process encodings.
+TeX is sometimes bad dealing with 8bit encodings and rather bad
+dealing with multibyte encodings.  So the process encoding output
+might need to get temporarily reprocessed into the original byte
+stream before the buffer characters can be identified.  XEmacs
+21.4 is rather bad at preserving incomplete multibyte characters
+in that process.  This variable makes it possible to use a
+reconstructable coding system in the run buffer instead.  Specify
+an alist of base coding system names here, which you can get
+using
+
+  \(coding-system-name (coding-system-base buffer-file-coding-system))
+
+in properly detected buffers."
+  :group 'preview-latex
+  :type '(repeat (cons symbol symbol)))
+
+(defun preview-buffer-recode-system (base)
+  "This is supposed to translate unrepresentable base encodings
+ into something that can be used safely for byte streams in the
+ run buffer.  XEmacs mule-ucs is so broken that this may be
+ needed."
+  (or (cdr (assq (coding-system-name base)
+		 preview-buffer-recoding-alist))
+      base))
 
 (defun preview-mode-setup ()
   "Setup proper buffer hooks and behavior for previews."
+  (set (make-local-variable 'desktop-save-buffer)
+       #'desktop-buffer-preview-misc-data)
   (mapc #'make-local-hook
         '(pre-command-hook post-command-hook
 	  before-change-functions after-change-functions))
@@ -492,17 +471,28 @@ Pure borderless black-on-white will return quadruple NIL."
   (add-hook 'before-change-functions #'preview-handle-before-change nil t)
   (add-hook 'after-change-functions #'preview-handle-after-change nil t)
   (easy-menu-add preview-menu)
+  (unless preview-tb-icon
+    (setq preview-tb-icon (preview-filter-specs
+				       preview-tb-icon-specs))
+    (when preview-tb-icon
+      (setq preview-tb-icon
+	    (vector
+	     (list preview-tb-icon)
+	     #'preview-at-point
+	     t
+	     "Preview on/off at point"))))
 ;;; [Courtesy Stephen J. Turnbull, with some modifications
 ;;;  Message-ID: <87el9fglsj.fsf@tleepslib.sk.tsukuba.ac.jp>
 ;;;  I could not have figured this out for the world]
 ;;; Hm, there really ought to be a way to get the spec that would be
 ;;; instantiated in a given domain
-  (let ((tb (cdadar (or (specifier-spec-list default-toolbar (current-buffer))
-			(specifier-spec-list default-toolbar 'global)))))
-    (unless (member preview-icon-toolbar-button tb)
-      (set-specifier default-toolbar
-		     (append tb (list preview-icon-toolbar-button))
-		     (current-buffer))))
+  (when preview-tb-icon
+    (let ((tb (cdadar (or (specifier-spec-list default-toolbar (current-buffer))
+			  (specifier-spec-list default-toolbar 'global)))))
+      (unless (member preview-tb-icon tb)
+	(set-specifier default-toolbar
+		       (append tb (list preview-tb-icon))
+		       (current-buffer)))))
   (when buffer-file-name
     (let* ((filename (expand-file-name buffer-file-name))
 	   format-cons)
@@ -557,19 +547,14 @@ overlays not in the active window."
   (let (newlist (pt (point)))
     (setq preview-temporary-opened
 	  (dolist (ov preview-temporary-opened newlist)
-	    (if (catch 'keep
-		  (unless (and (overlay-buffer ov)
-			       (eq (overlay-get ov 'preview-state) 'inactive))
-		    (throw 'keep nil))
-		  (unless (and (eq (overlay-buffer ov) (current-buffer))
-                               (not (extent-detached-p ov)))
-		    (throw 'keep t))
-		  (when (and (>= pt (overlay-start ov))
-			     (< pt (overlay-end ov)))
-		    (throw 'keep t))
-		  (preview-toggle ov t)
-		  nil)
-		(push ov newlist))))
+	    (and (extent-object ov)
+		 (not (extent-detached-p ov))
+		 (eq (extent-property ov 'preview-state) 'inactive)
+		 (if (and (eq (extent-object ov) (current-buffer))
+			  (or (<= pt (extent-start-position ov))
+			      (>= pt (extent-end-position ov))))
+		     (preview-toggle ov t)
+		   (push ov newlist)))))
     (if	(preview-auto-reveal-p preview-auto-reveal)
 	(map-extents #'preview-open-overlay nil
 		     pt pt nil nil 'preview-state 'active)
@@ -744,28 +729,18 @@ of an insertion."
 		    ov (extent-start-position ov) maparg)) nil
 		    end end beg 'end-in-region 'preview-state 'active)))
 
-(defun preview-export-image (image)
-  "Format an IMAGE into something printable."
-  (list (image-instance-file-name (glyph-image-instance image))
-	(glyph-image-type image)
-	(glyph-baseline-instance image)))
-
 (defun preview-import-image (image)
   "Convert the printable IMAGE rendition back to an image."
   (if (eq (car image) 'image)
       (let ((plist (cdr image)))
-	(preview-create-icon
+	(preview-create-icon-1
 	 (plist-get plist :file)
 	 (plist-get plist :type)
 	 (plist-get plist :ascent)))
-    (preview-create-icon (nth 0 image)
+    (preview-create-icon-1 (nth 0 image)
 			 (nth 1 image)
 			 (nth 2 image))))
 
-(defsubst preview-supports-image-type (imagetype)
-  "Return whether IMAGETYPE is supported by XEmacs."
-  (memq imagetype (image-instantiator-format-list)))
-
 (provide 'prv-xemacs)
 
 ;;; Local variables:
